#!/bin/bash

before_exit() {
  if [ "$is_success" == true ]; then
    echo "__SH__SCRIPT_END_SUCCESS__";
  else
    echo "__SH__SCRIPT_END_FAILURE__";
  fi
}

exec_cmd() {
  cmd=$@
  cmd_uuid=$(cat /proc/sys/kernel/random/uuid)
  cmd_start_timestamp=`date +"%s"`
  echo "__SH__CMD__START__|{\"type\":\"cmd\",\"sequenceNumber\":\"$cmd_start_timestamp\",\"id\":\"$cmd_uuid\"}|$cmd"
  eval $cmd
  cmd_status=$?
  cmd_end_timestamp=`date +"%s"`
  echo "__SH__CMD__END__|{\"type\":\"cmd\",\"sequenceNumber\":\"$cmd_start_timestamp\",\"id\":\"$cmd_uuid\",\"completed\":\"$cmd_status\"}|$cmd"
  return $cmd_status
}

exec_grp() {
  group_name=$1
  group_uuid=$(cat /proc/sys/kernel/random/uuid)
  group_start_timestamp=`date +"%s"`
  echo "__SH__GROUP__START__|{\"type\":\"grp\",\"sequenceNumber\":\"$group_start_timestamp\",\"id\":\"$group_uuid\"}|$group_name"
  eval "$group_name"
  group_status=$?
  group_end_timestamp=`date +"%s"`
  echo "__SH__GROUP__END__|{\"type\":\"grp\",\"sequenceNumber\":\"$group_end_timestamp\",\"id\":\"$group_uuid\",\"completed\":\"$group_status\"}|$group_name"
}
# Run mode
readonly RUN_MODE="production"

# Cluster node
readonly NODE_ID="2"
readonly NODE_TYPE_CODE=7000
readonly NODE_ARCHITECTURE="x86_64"
readonly NODE_OPERATING_SYSTEM="Ubuntu_18.04"
readonly NODE_DOCKER_VERSION="18.09"
readonly CLUSTER_TYPE_CODE=10023
readonly IS_RESTRICTED_NODE=""

# Subscription
readonly PROJECT_ID="1"

# Image
readonly EXEC_IMAGE="docker.bintray.io/jfrog/pipelines-u16reqproc:0.9.0"
readonly SHIPPABLE_RELEASE_VERSION="0.9.0"
readonly SHIPPABLE_RUNTIME_VERSION="master"

# API
readonly SHIPPABLE_API_URL="https://cpe-ci-api.jfrog.team"
readonly REQKICK_DOWNLOAD_URL="https://cpe-ci-api.jfrog.team/passthrough/repoArchives/reqKick.tar.gz"
readonly REPORTS_DOWNLOAD_URL="https://cpe-ci-api.jfrog.team/passthrough/binaries/reports/reports-x86_64-Ubuntu_18.04.tar.gz"

# WWW
readonly SHIPPABLE_WWW_URL="https://cpe-ci-pipelines.jfrog.team"

# AMQP
readonly SHIPPABLE_AMQP_URL="amqp://jaqowufapodayoki:yYtbhHRJUzqFo8or@10.166.10.16/shippable"
readonly SHIPPABLE_AMQP_DEFAULT_EXCHANGE="shippableEx"
readonly LISTEN_QUEUE="1.cluster_1"

# Scripts
readonly NODE_INIT_SCRIPT="x86_64/Ubuntu_18.04/Docker_18.09.sh"
readonly NODE_SCRIPTS_DOWNLOAD_LOCATION=/tmp/shippable/node.tar.gz
readonly NODE_SCRIPTS_LOCATION=/jfrog/nodeInit
readonly NODE_SCRIPTS_SUB_DIR=initScripts
readonly NODE_DATA_LOCATION=/jfrog/config
readonly NODE_LOGS_LOCATION="$NODE_DATA_LOCATION/logs"

# Swap
readonly IS_SWAP_ENABLED="false"

# Proxy
readonly SHIPPABLE_HTTP_PROXY=""
readonly SHIPPABLE_HTTPS_PROXY=""
readonly SHIPPABLE_NO_PROXY=""

# SSL
readonly NO_VERIFY_SSL="true"
check_dependencies() {
  is_success=false
  exec_cmd "echo 'Checking node dependencies'"

  command -v curl >/dev/null 2>&1
  local ret=$?
  if [ $? -ne 0 ]; then
    exec_cmd "echo 'curl not installed on host machine. Node requires curl to be installed'"
    is_success=false
  else
    exec_cmd "echo 'curl installed, continuing...'"
    exec_cmd "curl --version"
    is_success=true
  fi

  command -v tar >/dev/null 2>&1
  local ret=$?
  if [ $? -ne 0 ]; then
    exec_cmd "echo 'tar not installed on host machine. Node requires tar to be installed'"
    is_success=false
  else
    exec_cmd "echo 'tar installed, continuing...'"
    exec_cmd "tar --version"
    is_success=true
  fi
}

configure_proxy() {
  echo "Defaults        env_keep += \"http_proxy https_proxy no_proxy\"" > /etc/sudoers.d/shippable-proxy-whitelist

  local env_location="/etc/environment"
  if [ ! -z "$SHIPPABLE_HTTP_PROXY" ]; then
    http_proxy_env="export http_proxy=\"$SHIPPABLE_HTTP_PROXY\""
    http_proxy_exists=$(grep "^$http_proxy_env$" $env_location || echo "")
    if [ -z "$http_proxy_exists" ]; then
      echo "Adding http_proxy to /etc/environment"
      sed -i '/^http_proxy=/d' $env_location
      echo "$http_proxy_env" >> $env_location
    fi
  else
    http_proxy_exists=$(grep "^export http_proxy=" $env_location || echo "")
    if [ ! -z "$http_proxy_exists" ]; then
      echo "Removing http_proxy from /etc/environment"
      unset http_proxy
      sed -i '/^export http_proxy=/d' $env_location
    fi
  fi

  if [ ! -z "$SHIPPABLE_HTTPS_PROXY" ]; then
    https_env="export https_proxy=\"$SHIPPABLE_HTTPS_PROXY\""
    https_exists=$(grep "^$https_env$" $env_location || echo "")
    if [ -z "$https_exists" ]; then
      echo "Adding https_proxy to /etc/environment"
      sed -i '/^export https_proxy=/d' $env_location
      echo "$https_env" >> $env_location
    fi
  else
    https_exists=$(grep "^export https_proxy=" $env_location || echo "")
    if [ ! -z "$https_exists" ]; then
      echo "Removing https_proxy from /etc/environment"
      unset https_proxy
      sed -i '/^export https_proxy=/d' $env_location
    fi
  fi

  if [ ! -z "$SHIPPABLE_NO_PROXY" ]; then
    no_proxy_env="export no_proxy=\"$SHIPPABLE_NO_PROXY\""
    no_proxy_exists=$(grep "^$no_proxy_env$" $env_location || echo "")
    if [ -z "$no_proxy_exists" ]; then
      echo "Adding no_proxy to /etc/environment"
      sed -i '/^export no_proxy=/d' $env_location
      echo "$no_proxy_env" >> $env_location
    fi
  else
    no_proxy_exists=$(grep "^export no_proxy=" $env_location || echo "")
    if [ ! -z "$no_proxy_exists" ]; then
      echo "Removing no_proxy from /etc/environment"
      unset no_proxy
      sed -i '/^export no_proxy=/d' $env_location
    fi
  fi

  source $env_location
}

get_repo() {
  is_success=false
  exec_cmd "echo 'Downloading Shippable node init repo'"

  exec_cmd "mkdir -p $NODE_SCRIPTS_LOCATION"
  exec_cmd "mkdir -p $NODE_DATA_LOCATION"
  exec_cmd "mkdir -p $NODE_LOGS_LOCATION"
  exec_cmd "mkdir -p /tmp/shippable"

  exec_cmd "echo 'Pulling scripts from api'"
  local download_url="$SHIPPABLE_API_URL/passthrough/repoArchives/node.tar.gz"

  exec_cmd "curl -LkSs \
    --connect-timeout 60 \
    --max-time 120 \
    '$download_url' \
    -o $NODE_SCRIPTS_DOWNLOAD_LOCATION"

  exec_cmd "echo 'Un-taring Shippable node init repo'"
  exec_cmd "tar -xvzf \
    '$NODE_SCRIPTS_DOWNLOAD_LOCATION' \
    -C $NODE_SCRIPTS_LOCATION \
    --strip-components=1"

  is_success=true
}

update_envs() {
  is_success=false

  local node_env_template=$NODE_SCRIPTS_LOCATION/usr/_node.env.template
  local node_env=$NODE_DATA_LOCATION/nodeInit.env

  if [ ! -f "$node_env_template" ]; then
    exec_cmd "echo 'Node environment template file not found: $node_env_template'"
    is_success=false
    return
  else
    exec_cmd "echo 'Node environment template file found: $node_env_template'"
  fi

  exec_cmd "echo 'Writing node specific envs to $node_env'"

  sed "s#{{NODE_ID}}#$NODE_ID#g" $node_env_template > $node_env
  sed -i "s#{{EXEC_IMAGE}}#$EXEC_IMAGE#g" $node_env
  sed -i "s#{{IS_SWAP_ENABLED}}#$IS_SWAP_ENABLED#g" $node_env
  sed -i "s#{{LISTEN_QUEUE}}#$LISTEN_QUEUE#g" $node_env
  sed -i "s#{{NODE_ARCHITECTURE}}#$NODE_ARCHITECTURE#g" $node_env
  sed -i "s#{{NODE_DOCKER_VERSION}}#$NODE_DOCKER_VERSION#g" $node_env
  sed -i "s#{{NODE_SCRIPTS_LOCATION}}#$NODE_SCRIPTS_LOCATION#g" $node_env
  sed -i "s#{{NODE_INIT_SCRIPT}}#$NODE_INIT_SCRIPT#g" $node_env
  sed -i "s#{{NODE_OPERATING_SYSTEM}}#$NODE_OPERATING_SYSTEM#g" $node_env
  sed -i "s#{{NODE_TYPE_CODE}}#$NODE_TYPE_CODE#g" $node_env
  sed -i "s#{{RUN_MODE}}#$RUN_MODE#g" $node_env
  sed -i "s#{{SHIPPABLE_AMQP_DEFAULT_EXCHANGE}}#$SHIPPABLE_AMQP_DEFAULT_EXCHANGE#g" $node_env
  sed -i "s#{{SHIPPABLE_AMQP_URL}}#$SHIPPABLE_AMQP_URL#g" $node_env
  sed -i "s#{{SHIPPABLE_API_URL}}#$SHIPPABLE_API_URL#g" $node_env
  sed -i "s#{{SHIPPABLE_WWW_URL}}#$SHIPPABLE_WWW_URL#g" $node_env
  sed -i "s#{{SHIPPABLE_RUNTIME_VERSION}}#$SHIPPABLE_RUNTIME_VERSION#g" $node_env
  sed -i "s#{{SHIPPABLE_RELEASE_VERSION}}#$SHIPPABLE_RELEASE_VERSION#g" $node_env
  sed -i "s#{{PROJECT_ID}}#$PROJECT_ID#g" $node_env
  sed -i "s#{{REQKICK_DOWNLOAD_URL}}#$REQKICK_DOWNLOAD_URL#g" $node_env
  sed -i "s#{{SHIPPABLE_HTTP_PROXY}}#$SHIPPABLE_HTTP_PROXY#g" $node_env
  sed -i "s#{{SHIPPABLE_HTTPS_PROXY}}#$SHIPPABLE_HTTPS_PROXY#g" $node_env
  sed -i "s#{{SHIPPABLE_NO_PROXY}}#$SHIPPABLE_NO_PROXY#g" $node_env
  sed -i "s#{{REPORTS_DOWNLOAD_URL}}#$REPORTS_DOWNLOAD_URL#g" $node_env
  sed -i "s#{{NO_VERIFY_SSL}}#$NO_VERIFY_SSL#g" $node_env
  sed -i "s#{{CLUSTER_TYPE_CODE}}#$CLUSTER_TYPE_CODE#g" $node_env
  sed -i "s#{{IS_RESTRICTED_NODE}}#$IS_RESTRICTED_NODE#g" $node_env

  exec_cmd "echo 'Successfully updated node specific envs to $node_env'"

  is_success=true
}

initialize() {
  is_success=false

  exec_cmd "echo 'Initializing node'"
  exec_cmd "/bin/bash $NODE_SCRIPTS_LOCATION/$NODE_SCRIPTS_SUB_DIR/$NODE_ARCHITECTURE/$NODE_OPERATING_SYSTEM/boot.sh"

  is_success=true
}

trap before_exit EXIT
exec_grp "check_dependencies"

if [ ! -z "$SHIPPABLE_HTTP_PROXY" ] || [ ! -z "$SHIPPABLE_HTTPS_PROXY" ] || [ ! -z "$SHIPPABLE_NO_PROXY" ]; then
  trap before_exit EXIT
  exec_grp "configure_proxy"
fi

trap before_exit EXIT
exec_grp "get_repo"

trap before_exit EXIT
exec_grp "update_envs"

trap before_exit EXIT
exec_grp "initialize"
